@@ -26,7 +26,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-language-manager.c 5855 2007-08-27 18:13:10Z paolo $
+ * $Id: gedit-language-manager.c 6351 2008-08-06 17:45:35Z jessevdk $
  */
 
 #include <string.h>
@@ -49,8 +49,6 @@ gedit_get_language_manager (void)
 	return language_manager;
 }
 
-
-
 static gint
 language_compare (gconstpointer a, gconstpointer b)
 {
@@ -68,8 +66,11 @@ gedit_language_manager_list_languages_sorted (GtkSourceLanguageManager *lm,
 {
 	GSList *languages = NULL;
 	const gchar * const *ids;
-	
+
 	ids = gtk_source_language_manager_get_language_ids (lm);
+	if (ids == NULL)
+		return NULL;
+
 	while (*ids != NULL)
 	{
 		GtkSourceLanguage *lang;
@@ -79,7 +80,9 @@ gedit_language_manager_list_languages_sorted (GtkSourceLanguageManager *lm,
 		++ids;
 
 		if (include_hidden || !gtk_source_language_get_hidden (lang))
+		{
 			languages = g_slist_prepend (languages, lang);
+		}
 	}
 
 	return g_slist_sort (languages, (GCompareFunc)language_compare);
@@ -108,7 +111,7 @@ get_languages_cache (GtkSourceLanguageManager *lm)
 					 res,
 					 (GDestroyNotify)g_hash_table_unref);
 	}
-	
+
 	return res;
 }
 
@@ -155,6 +158,8 @@ gedit_language_manager_get_language_from_mime_type (GtkSourceLanguageManager *lm
 			     "Cache miss for %s", mime_type);
 
 	languages = gtk_source_language_manager_get_language_ids (lm);
+	if (languages == NULL)
+		return NULL;
 
 	while (*languages != NULL)
 	{
@@ -172,15 +177,7 @@ gedit_language_manager_get_language_from_mime_type (GtkSourceLanguageManager *lm
 
 		for (i = 0; mime_types[i] != NULL; ++i)
 		{
-			GnomeVFSMimeEquivalence res;
-
-			/* Use gnome_vfs_mime_type_get_equivalence instead of
-			 * strcmp, to take care of mime-types inheritance
-			 * (see bug #324191) */
-			res = gnome_vfs_mime_type_get_equivalence (mime_type, 
-								   mime_types[i]);
-
-			if (res == GNOME_VFS_MIME_IDENTICAL)
+			if (g_content_type_equals(mime_type, mime_types[i]))
 			{
 				/* If the mime-type of lang is identical to "mime-type" then
 				   return lang */
@@ -192,7 +189,7 @@ gedit_language_manager_get_language_from_mime_type (GtkSourceLanguageManager *lm
 
 				break;
 			}
-			else if ((res == GNOME_VFS_MIME_PARENT) && (parent == NULL))
+			else if (parent == NULL && g_content_type_is_a (mime_type, mime_types[i]))
 			{
 				/* If the mime-type of lang is a parent of "mime-type" then
 				   remember it. We will return it if we don't find
