@@ -21,12 +21,11 @@ import shutil
 
 import gobject
 import gtk
-from gtk import glade
 from gtk import gdk
 import gtksourceview2 as gsv
 import pango
 import gedit
-import gnomevfs
+import gio
 
 from Snippet import Snippet
 from Helper import *
@@ -174,7 +173,8 @@ class Manager:
                 
 
         def on_tree_view_drag_data_get(self, widget, context, selection_data, info, time):
-                selection_data.set_uris(['file://' + gnomevfs.escape_path_string(self._temp_export)])
+                gfile = gio.File(self._temp_export)
+                selection_data.set_uris([gfile.get_uri()])
        
         def on_tree_view_drag_begin(self, widget, context):
                 self.dragging = True
@@ -282,42 +282,14 @@ class Manager:
                         self.manager.set_search_path(dirs + self.manager.get_search_path())
                 
                 return self.manager
-
-        def custom_handler(self, xml, function_name, widget_name, str1, str2, \
-                        int1 , int2):
-                if function_name == 'create_source_view':
-                        # Create the buffer, view and reset to defaults
-                        buf = gedit.Document()
-                        #buf.set_style_scheme(None)
-                        buf.set_check_brackets(False)
-                        
-                        source_view = gedit.View(buf)
-                        source_view.set_auto_indent(True)
-                        source_view.set_insert_spaces_instead_of_tabs(False)
-                        source_view.set_smart_home_end(gsv.SMART_HOME_END_AFTER)
-                        source_view.set_tabs_width(2)
-                        source_view.set_highlight_current_line(False)
-                        source_view.set_show_margin(False)
-                        source_view.set_show_line_numbers(False)
-                        source_view.set_font(True, "Monospace 10")
-                        
-                        manager = self.get_language_manager()
-                        lang = manager.get_language_by_id('snippets')
-                        
-                        if lang:
-                                buf.set_highlight(True)
-                                buf.set_language(lang)
-                                self.snippets_doc = Document(None, source_view)
-
-                        return source_view
-                else:
-                        return None
                 
         def build(self):
-                glade.set_custom_handler(self.custom_handler)
-                self.xml = glade.XML(os.path.dirname(__file__) + '/snippets.glade')
+                self.builder = gtk.Builder()
+                self.builder.add_from_file(os.path.dirname(__file__) + '/snippets.ui')
                 
-                handlers_dic = {'on_dialog_snippets_response': self.on_dialog_snippets_response,
+                handlers_dic = {
+                        'on_dialog_snippets_response': self.on_dialog_snippets_response,
+                        'on_dialog_snippets_destroy': self.on_dialog_snippets_destroy,
                         'on_button_new_snippet_clicked': self.on_button_new_snippet_clicked,
                         'on_button_import_snippets_clicked': self.on_button_import_snippets_clicked,
                         'on_button_export_snippets_clicked': self.on_button_export_snippets_clicked,
@@ -331,7 +303,7 @@ class Manager:
                         'on_tree_view_snippets_row_expanded': self.on_tree_view_snippets_row_expanded,
                         'on_tree_view_snippets_key_press': self.on_tree_view_snippets_key_press}
 
-                self.xml.signal_autoconnect(handlers_dic)
+                self.builder.connect_signals(handlers_dic)
                 
                 self.build_tree_view()
                 self.build_model()
@@ -341,7 +313,14 @@ class Manager:
 
                 source_view = self['source_view_snippet']
                 source_view.modify_font(pango.FontDescription('Monospace 8'))
+                manager = self.get_language_manager()
+                lang = manager.get_language('snippets')
 
+                if lang:
+                        source_view.get_buffer().set_highlight_syntax(True)
+                        source_view.get_buffer().set_language(lang)
+                        self.snippets_doc = Document(None, source_view)
+                                
                 entry = self['combo_drop_targets'].child
                 entry.connect('focus-out-event', self.on_entry_drop_targets_focus_out)
                 entry.connect('drag-data-received', self.on_entry_drop_targets_drag_data_received)
@@ -353,7 +332,7 @@ class Manager:
                 self.dlg = self['dialog_snippets']
         
         def __getitem__(self, key):
-                return self.xml.get_widget(key)
+                return self.builder.get_object(key)
 
         def is_filled(self, piter):
                 if not self.model.iter_has_child(piter):
@@ -612,11 +591,7 @@ class Manager:
                         piter = self.model.iter_next(piter)
 
         # Callbacks
-        def on_dialog_snippets_response(self, dlg, resp):
-                if resp == gtk.RESPONSE_HELP:
-                        gedit.help_display(self.dlg, 'gedit.xml', 'gedit-snippets-plugin')
-                        return
-
+        def on_dialog_snippets_destroy(self, dlg):
                 # Remove temporary drag export
                 if self._temp_export:
                       shutil.rmtree(os.path.dirname(self._temp_export))
@@ -630,8 +605,14 @@ class Manager:
                 self.unref_languages()        
                 self.snippet = None        
                 self.model = None
+                self.dlg = None                
+        
+        def on_dialog_snippets_response(self, dlg, resp):                                
+                if resp == gtk.RESPONSE_HELP:
+                        gedit.help_display(self.dlg, 'gedit', 'gedit-snippets-plugin')
+                        return
+
                 self.dlg.destroy()
-                self.dlg = None
         
         def on_cell_editing_started(self, renderer, editable, path):
                 piter = self.model.get_iter(path)
@@ -777,7 +758,9 @@ class Manager:
                                 continue
 
                         # Remove file://
-                        filename = gnomevfs.get_local_path_from_uri(filename)
+                        gfile = gio.File(filename)
+                        filename = gfile.get_path()
+
                         importer = Importer(filename)
                         error = importer.run()
          
@@ -1148,7 +1131,7 @@ class Manager:
                 
                 for uri in uris:
                         try:
-                                mime = gnomevfs.get_mime_type(uri)
+                                mime = gio.content_type_guess(uri)
                         except:
                                 mime = None
                         
